@@ -1,10 +1,11 @@
 from . import io, log
+from ._context_vars import StrictContextVar
 from .aggregate import aggregate, aggregate_async
 from .backoff import asyncbackoff, asyncretry
 from .circuit_breaker import CircuitBreaker, CircuitBroken, cutout
 from .context import Context, get_context
 from .counters import Statistic, get_statistics
-from .entrypoint import Entrypoint, entrypoint, run
+from .entrypoint import CURRENT_ENTRYPOINT, Entrypoint, entrypoint, run
 from .iterator_wrapper import IteratorWrapper
 from .periodic import PeriodicCallback
 from .plugins import plugins
@@ -30,6 +31,7 @@ from .worker_pool import WorkerPool
 
 
 __all__ = (
+    "CURRENT_ENTRYPOINT",
     "CircuitBreaker",
     "CircuitBroken",
     "Context",
@@ -47,6 +49,7 @@ __all__ = (
     "StrategyException",
     "StrategySkip",
     "StrategyStop",
+    "StrictContextVar",
     "ThreadPoolExecutor",
     "WorkerPool",
     "__version__",
