@@ -2,7 +2,8 @@ import asyncio
 import logging
 import os
 import socket
-from typing import Optional
+import sys
+from typing import Any, Iterator, Optional
 
 from ._context_vars import EVENT_LOOP
 
@@ -17,13 +18,56 @@ except ImportError:
     def time_ns() -> int:
         return int(time() * 1000000000)
 
-
 try:
     from typing import final
 except ImportError:
     from typing_extensions import final  # type: ignore
 
 
+try:
+    from typing import TypeAlias
+except ImportError:
+    from typing_extensions import TypeAlias
+
+
+if sys.version_info >= (3, 10):
+    from typing import ParamSpec
+else:
+    from typing_extensions import ParamSpec
+
+
+if sys.version_info >= (3, 8):
+    from typing import Protocol
+else:
+    from typing_extensions import Protocol
+
+
+class EntrypointProtocol(Protocol):
+    @property
+    def name(self) -> str:
+        ...
+
+    def load(self) -> Any:
+        ...
+
+
+# noinspection PyUnresolvedReferences
+try:
+    from importlib.metadata import Distribution, EntryPoint
+
+    def entry_pont_iterator(entry_point: str) -> Iterator[EntrypointProtocol]:
+        ep: EntryPoint
+        for dist in Distribution.discover():
+            for ep in dist.entry_points:
+                if ep.group == entry_point:
+                    yield ep
+except ImportError:
+    import pkg_resources
+
+    def entry_pont_iterator(entry_point: str) -> Iterator[EntrypointProtocol]:
+        yield from pkg_resources.iter_entry_points(entry_point)
+
+
 class EventLoopMixin:
     __slots__ = "_loop",
 
@@ -76,7 +120,12 @@ set_current_loop = EVENT_LOOP.set
 get_current_loop = EVENT_LOOP.get
 
 __all__ = (
+    "EntrypointProtocol",
     "EventLoopMixin",
+    "ParamSpec",
+    "Protocol",
+    "TypeAlias",
+    "entry_pont_iterator",
     "event_loop_policy",
     "final",
     "get_current_loop",
