@@ -6,20 +6,23 @@ import threading
 import time
 import warnings
 from concurrent.futures import ThreadPoolExecutor as ThreadPoolExecutorBase
+from dataclasses import dataclass, field
 from functools import partial, wraps
 from multiprocessing import cpu_count
 from queue import SimpleQueue
 from types import MappingProxyType
 from typing import (
-    Any, Awaitable, Callable, Coroutine, Dict, FrozenSet, NamedTuple, Optional,
-    Set, Tuple, TypeVar,
+    Any, Awaitable, Callable, Coroutine, Dict, FrozenSet, Optional, Set, Tuple,
+    TypeVar,
 )
 
 from ._context_vars import EVENT_LOOP
+from .compat import ParamSpec
 from .counters import Statistic
 from .iterator_wrapper import IteratorWrapper
 
 
+P = ParamSpec("P")
 T = TypeVar("T")
 F = TypeVar("F", bound=Callable[..., Any])
 log = logging.getLogger(__name__)
@@ -29,6 +32,10 @@ def context_partial(
     func: F, *args: Any,
     **kwargs: Any,
 ) -> Any:
+    warnings.warn(
+        "context_partial has been deprecated and will be removed",
+        DeprecationWarning,
+    )
     context = contextvars.copy_context()
     return partial(context.run, func, *args, **kwargs)
 
@@ -37,14 +44,6 @@ class ThreadPoolException(RuntimeError):
     pass
 
 
-class WorkItemBase(NamedTuple):
-    func: Callable[..., Any]
-    args: Tuple[Any, ...]
-    kwargs: Dict[str, Any]
-    future: asyncio.Future
-    loop: asyncio.AbstractEventLoop
-
-
 class ThreadPoolStatistic(Statistic):
     threads: int
     done: int
@@ -54,48 +53,119 @@ class ThreadPoolStatistic(Statistic):
     sum_time: float
 
 
+@dataclass(frozen=True)
+class WorkItemBase:
+    func: Callable[..., Any]
+    statistic: ThreadPoolStatistic
+    future: asyncio.Future
+    loop: asyncio.AbstractEventLoop
+    args: Tuple[Any, ...] = field(default_factory=tuple)
+    kwargs: Dict[str, Any] = field(default_factory=dict)
+    context: contextvars.Context = field(
+        default_factory=contextvars.copy_context,
+    )
+
+
+def _set_workitem_result(
+    future: asyncio.Future, result: Optional[Any],
+    exception: Optional[BaseException],
+) -> Any:
+    if future.done():
+        return
+
+    if exception:
+        future.set_exception(exception)
+    else:
+        future.set_result(result)
+
+
 class WorkItem(WorkItemBase):
-    @staticmethod
-    def set_result(
-        future: asyncio.Future, result: Any, exception: Exception,
-    ) -> None:
-        if future.done():
-            return
 
-        if exception:
-            future.set_exception(exception)
-        else:
-            future.set_result(result)
-
-    def __call__(self, statistic: ThreadPoolStatistic) -> None:
+    def __call__(self, no_return: bool = False) -> None:
         if self.future.done():
             return
 
-        result, exception = None, None
+        if self.loop.is_closed():
+            log.warning("Event loop is closed. Ignoring %r", self.func)
+            raise asyncio.CancelledError
 
-        delta = -self.loop.time()
+        result, exception = None, None
+        delta = -time.monotonic()
         try:
-            result = self.func(*self.args, **self.kwargs)
-            statistic.success += 1
+            result = self.context.run(
+                self.func, *self.args, **self.kwargs,
+            )
+            self.statistic.success += 1
         except BaseException as e:
-            statistic.error += 1
+            self.statistic.error += 1
             exception = e
         finally:
             delta += time.monotonic()
-            statistic.sum_time += delta
-            statistic.done += 1
+            self.statistic.sum_time += delta
+            self.statistic.done += 1
+
+        if no_return:
+            return
 
         if self.loop.is_closed():
+            log.warning(
+                "Event loop is closed. Forget execution result for %r",
+                self.func,
+            )
             raise asyncio.CancelledError
 
         self.loop.call_soon_threadsafe(
-            self.__class__.set_result,
+            _set_workitem_result,
             self.future,
             result,
             exception,
         )
 
 
+class TaskChannelCloseException(RuntimeError):
+    pass
+
+
+class TaskChannel(SimpleQueue):
+    closed_event: threading.Event
+
+    def __init__(self) -> None:
+        super().__init__()
+        self.closed_event = threading.Event()
+
+    def get(self, *args: Any, **kwargs: Any) -> WorkItem:
+        if self.closed_event.is_set():
+            raise TaskChannelCloseException()
+
+        item: Optional[WorkItem] = super().get(*args, **kwargs)
+        if item is None:
+            self.put(None)
+            raise TaskChannelCloseException()
+        return item
+
+    def close(self) -> None:
+        self.closed_event.set()
+        self.put(None)
+
+
+def thread_pool_thread_loop(
+    tasks: TaskChannel,
+    statistic: ThreadPoolStatistic,
+    stop_event: threading.Event,
+    pool_shutdown_event: threading.Event,
+) -> None:
+    statistic.threads += 1
+
+    try:
+        while not pool_shutdown_event.is_set():
+            tasks.get()()
+    except (TaskChannelCloseException, asyncio.CancelledError):
+        return None
+    finally:
+        stop_event.set()
+        statistic.threads -= 1
+
+
 class ThreadPoolExecutor(ThreadPoolExecutorBase):
     __slots__ = (
         "__futures", "__pool", "__tasks",
@@ -103,30 +173,31 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
     )
 
     DEFAULT_POOL_SIZE = min((max((cpu_count() or 1, 4)), 32))
+    SHUTDOWN_TIMEOUT = 10
 
     def __init__(
         self, max_workers: int = DEFAULT_POOL_SIZE,
-        loop: Optional[asyncio.AbstractEventLoop] = None,
         statistic_name: Optional[str] = None,
     ) -> None:
-        """"""
-        if loop:
-            warnings.warn(DeprecationWarning("loop argument is obsolete"))
-
         self.__futures: Set[asyncio.Future[Any]] = set()
-
         self.__thread_events: Set[threading.Event] = set()
-        self.__tasks: SimpleQueue[Optional[WorkItem]] = SimpleQueue()
+        self.__tasks = TaskChannel()
         self.__write_lock = threading.RLock()
+        self.__max_workers = max_workers
+        self.__shutdown_event = threading.Event()
         self._statistic = ThreadPoolStatistic(statistic_name)
 
-        pools = set()
-        for idx in range(max_workers):
-            pools.add(self._start_thread(idx))
+        threads = set()
+        # starting minimum threads as possible
+        for idx in range(2 if max_workers > 1 else 1):
+            threads.add(self._start_thread(idx))
 
-        self.__pool: FrozenSet[threading.Thread] = frozenset(pools)
+        self.__pool: FrozenSet[threading.Thread] = frozenset(threads)
 
     def _start_thread(self, idx: int) -> threading.Thread:
+        if self.__shutdown_event.is_set():
+            raise RuntimeError("Can not create a thread after shutdown")
+
         event = threading.Event()
         self.__thread_events.add(event)
 
@@ -136,40 +207,20 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
             thread_name += f" from pool {self._statistic.name}"
 
         thread = threading.Thread(
-            target=self._in_thread,
+            target=thread_pool_thread_loop,
             name=thread_name.strip(),
-            args=(event,),
+            daemon=True,
+            args=(
+                self.__tasks,
+                self._statistic,
+                event,
+                self.__shutdown_event,
+            ),
         )
 
-        thread.daemon = True
         thread.start()
         return thread
 
-    def _in_thread(self, event: threading.Event) -> None:
-        self._statistic.threads += 1
-
-        try:
-            while True:
-                work_item = self.__tasks.get()
-
-                if work_item is None:
-                    break
-
-                try:
-                    if work_item.loop.is_closed():
-                        log.warning(
-                            "Event loop is closed. Call %r skipped",
-                            work_item.func,
-                        )
-                        continue
-
-                    work_item(self._statistic)
-                finally:
-                    del work_item
-        finally:
-            self._statistic.threads -= 1
-            event.set()
-
     def submit(  # type: ignore
         self, fn: F, *args: Any, **kwargs: Any,
     ) -> asyncio.Future:
@@ -179,44 +230,84 @@ class ThreadPoolExecutor(ThreadPoolExecutorBase):
         if fn is None or not callable(fn):
             raise ValueError("First argument must be callable")
 
-        loop = asyncio.get_event_loop()
-        future: asyncio.Future = loop.create_future()
-        self.__futures.add(future)
-        future.add_done_callback(self.__futures.remove)
-
         with self.__write_lock:
+            if self.__shutdown_event.is_set():
+                raise RuntimeError("Pool is shutdown")
+
+            if (
+                len(self.__pool) < self.__max_workers and
+                len(self.__futures) >= len(self.__pool)
+            ):
+                self._adjust_thread_count()
+
+            loop = asyncio.get_event_loop()
+            future: asyncio.Future = loop.create_future()
+            self.__futures.add(future)
+            future.add_done_callback(self.__futures.discard)
+
             self.__tasks.put_nowait(
                 WorkItem(
-                    func=fn,
-                    args=args,
-                    kwargs=kwargs,
-                    future=future,
-                    loop=loop,
+                    func=fn, args=args, kwargs=kwargs, loop=loop,
+                    future=future, statistic=self._statistic,
                 ),
             )
 
-        self._statistic.submitted += 1
-        return future
+            self._statistic.submitted += 1
+            return future
 
     # noinspection PyMethodOverriding
     def shutdown(self, wait: bool = True) -> None:  # type: ignore
-        for _ in self.__pool:
-            self.__tasks.put_nowait(None)
+        with self.__write_lock:
+            if self.__shutdown_event.is_set():
+                return None
 
-        for f in filter(lambda x: not x.done(), self.__futures):
-            f.set_exception(ThreadPoolException("Pool closed"))
+            self.__shutdown_event.set()
 
-        if not wait:
-            return
+            del self.__pool
 
-        while not all(e.is_set() for e in self.__thread_events):
-            time.sleep(0)
+            futures = self.__futures
+            del self.__futures
+
+            thread_events = self.__thread_events
+            del self.__thread_events
+
+            self.__tasks.close()
+
+            while futures:
+                future = futures.pop()
+                if future.done():
+                    continue
+                future.set_exception(ThreadPoolException("Pool closed"))
+
+            if not wait:
+                return None
+
+            start_time = time.monotonic()
+            while not all(e.is_set() for e in thread_events):
+                time.sleep(0.01)
+                if (time.monotonic() - start_time) > self.SHUTDOWN_TIMEOUT:
+                    log.warning(
+                        "Waiting for shutting down the pool %r "
+                        "cancelled due to timeout",
+                        self,
+                    )
+                    return None
 
     def _adjust_thread_count(self) -> None:
-        raise NotImplementedError
+        pool_size = len(self.__pool)
+        new_threads_count = (
+            min(pool_size * 2, self.__max_workers) - pool_size
+        )
+        if new_threads_count <= 0:
+            return None
+
+        threads = []
+        for idx in range(pool_size, pool_size + new_threads_count):
+            threads.append(self._start_thread(idx))
+        self.__pool = frozenset(list(self.__pool) + threads)
 
     def __del__(self) -> None:
-        self.shutdown()
+        self.__tasks.close()
 
 
 def run_in_executor(
@@ -228,7 +319,7 @@ def run_in_executor(
     try:
         loop = asyncio.get_running_loop()
         return loop.run_in_executor(
-            executor, context_partial(func, *args, **kwargs),
+            executor, partial(func, *args, **kwargs),
         )
     except RuntimeError:
         # In case the event loop is not running right now is
@@ -236,7 +327,7 @@ def run_in_executor(
         async def lazy_wrapper() -> T:
             loop = asyncio.get_running_loop()
             return await loop.run_in_executor(
-                executor, context_partial(func, *args, **kwargs),
+                executor, partial(func, *args, **kwargs),
             )
         return lazy_wrapper()
 
@@ -252,8 +343,8 @@ async def _awaiter(future: asyncio.Future) -> T:
 
 
 def threaded(
-    func: Callable[..., T],
-) -> Callable[..., Awaitable[T]]:
+    func: Callable[P, T],
+) -> Callable[P, Awaitable[T]]:
     if asyncio.iscoroutinefunction(func):
         raise TypeError("Can not wrap coroutine")
 
@@ -262,7 +353,7 @@ def threaded(
 
     @wraps(func)
     def wrap(
-        *args: Any, **kwargs: Any,
+        *args: P.args, **kwargs: P.kwargs,
     ) -> Awaitable[T]:
         return run_in_executor(func=func, args=args, kwargs=kwargs)
 
@@ -281,51 +372,23 @@ def run_in_new_thread(
     future = loop.create_future()
 
     statistic = ThreadPoolStatistic(statistic_name)
-
-    def set_result(result: Any) -> None:
-        if future.done() or loop.is_closed():
-            return
-
-        future.set_result(result)
-
-    def set_exception(exc: Exception) -> None:
-        if future.done() or loop.is_closed():
-            return
-
-        future.set_exception(exc)
-
-    @wraps(func)
-    def in_thread(target: F) -> None:
-        statistic.threads += 1
-        statistic.submitted += 1
-
-        try:
-            loop.call_soon_threadsafe(
-                set_result, target(),
-            )
-            statistic.success += 1
-        except Exception as exc:
-            statistic.error += 1
-            if loop.is_closed() and no_return:
-                return
-
-            elif loop.is_closed():
-                log.exception("Uncaught exception from separate thread")
-                return
-
-            loop.call_soon_threadsafe(set_exception, exc)
-        finally:
-            statistic.done += 1
-            statistic.threads -= 1
+    statistic.threads += 1
 
     thread = threading.Thread(
-        target=in_thread, name=func.__name__,
-        args=(
-            context_partial(func, *args, **kwargs),
+        target=WorkItem(
+            func=func,
+            args=args,
+            kwargs=kwargs,
+            loop=loop,
+            future=future,
+            statistic=statistic,
         ),
+        name=func.__name__,
+        kwargs=dict(no_return=no_return),
         daemon=detach,
     )
-    thread.start()
+    statistic.submitted += 1
+    loop.call_soon(thread.start)
     return future
 
 
@@ -361,7 +424,7 @@ def threaded_iterable(
     @wraps(func)
     def wrap(*args: Any, **kwargs: Any) -> Any:
         return IteratorWrapper(
-            context_partial(func, *args, **kwargs),  # type: ignore
+            partial(func, *args, **kwargs),
             max_size=max_size,
         )
 
@@ -385,7 +448,7 @@ def threaded_iterable_separate(
     @wraps(func)
     def wrap(*args: Any, **kwargs: Any) -> Any:
         return IteratorWrapperSeparate(
-            context_partial(func, *args, **kwargs),  # type: ignore
+            partial(func, *args, **kwargs),
             max_size=max_size,
         )
 
