@@ -51,6 +51,8 @@ class WorkerPool:
     worker_ids: Tuple[bytes, ...]
     pids: Set[int]
 
+    SERVER_CLOSE_TIMEOUT = 1
+
     if hasattr(socket, "AF_UNIX"):
         def _create_socket(self) -> None:
             path = mktemp(suffix=".sock", prefix="worker-")
@@ -80,7 +82,7 @@ class WorkerPool:
         process.kill()
 
     @threaded
-    def __create_supervisor(self, *identity: str) -> Popen:
+    def __create_supervisor(self, *identity: bytes) -> Popen:
         if self.__closing:
             raise RuntimeError("Pool closed")
 
@@ -240,7 +242,7 @@ class WorkerPool:
                     raise
         finally:
             self._statistic.processes -= 1
-            self.pids.remove(pid)
+            self.pids.discard(pid)
 
     def __start_handler(
         self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter,
@@ -249,7 +251,7 @@ class WorkerPool:
 
     def __task_add(self, task: asyncio.Task) -> None:
         self._statistic.task_added += 1
-        task.add_done_callback(self.__task_store.remove)
+        task.add_done_callback(self.__task_store.discard)
         self.__task_store.add(task)
 
     def __task(self, coroutine: Coroutine) -> asyncio.Task:
@@ -273,7 +275,7 @@ class WorkerPool:
     def __create_future(self) -> asyncio.Future:
         future = self.loop.create_future()
         self.__futures.add(future)
-        future.add_done_callback(self.__futures.remove)
+        future.add_done_callback(self.__futures.discard)
         return future
 
     def __reject_futures(self) -> None:
@@ -281,15 +283,20 @@ class WorkerPool:
 
     @shield
     async def close(self) -> None:
-        async with self.__closing_lock:
+        async with (self.__closing_lock):
             if self.__closing:
                 return
 
             self._kill_supervisor()
             self.__closing = True
             self.server.close()
-            await self.server.wait_closed()
-
+            await asyncio.gather(
+                asyncio.wait_for(
+                    self.server.wait_closed(),
+                    timeout=self.SERVER_CLOSE_TIMEOUT,
+                ),
+                return_exceptions=True,
+            )
             await cancel_tasks(tuple(self.__task_store))
             await cancel_tasks(tuple(self.__futures))
 
